#include "ObjReader.h"
#include "gzstream.h"
#include "Triangle3.h"
#include "IntersectableWrapper.h"


ObjReader::ObjReader()
{}


ObjReader::~ObjReader()
{}


bool ObjReader::LoadFile(const string &filename,
						 GtpVisibilityPreprocessor::ObjectContainer &objects) const
{
	igzstream samplesIn(filename.c_str());
	
	if (!samplesIn.is_open())
		return false;
	
	// read in triangle size
	int numTriangles;

	samplesIn.read(reinterpret_cast<char *>(&numTriangles), sizeof(int));
	objects.reserve(numTriangles);
	
	while (1)
	{
		GtpVisibilityPreprocessor::Triangle3 tri;
		
		samplesIn.read(reinterpret_cast<char *>(tri.mVertices + 0), 
					   sizeof(GtpVisibilityPreprocessor::Vector3));
		samplesIn.read(reinterpret_cast<char *>(tri.mVertices + 1), 
					   sizeof(GtpVisibilityPreprocessor::Vector3));
		samplesIn.read(reinterpret_cast<char *>(tri.mVertices + 2), 
					   sizeof(GtpVisibilityPreprocessor::Vector3));

		// end of file reached
		if (samplesIn.eof())
			break;

		GtpVisibilityPreprocessor::TriangleIntersectable *obj = 
			new GtpVisibilityPreprocessor::TriangleIntersectable(tri);
		objects.push_back(obj);
	}
	
	return true;
}


const std::vector<Ogre::Entity *> &ObjReader::GetObjects() const
{
	return mObjects;
}


#if TODO

Ogre::Entity *IVReader::createEntity(Ogre::SceneManager* sceneMgr, 
									 std::string name, 
									 ObjMeshData *mData, 
									 Ogre::Vector3 *translation)
{
	using namespace Ogre;

	std::string meshName = name.substr(name.find('/',0), name.length()) + "/Mesh";
	std::string entityName = name + "/Entity";

	Mesh* pMesh = NULL;
	if ((pMesh = (Mesh *) MeshManager::getSingleton().getByName(meshName).getPointer()) == NULL)
	{

		ObjMeshData *data = mData->expand();

		ObjManualMeshLoader *loader	= new ObjManualMeshLoader(data);

		if (translation != NULL) *translation = data->boundingBox->getCenter();

		addMeshToList(meshName, loader);

		Mesh* pMesh = MeshManager::getSingleton().createManual(meshName, "IVGroup", loader).getPointer(); 

		pMesh->load(); 
		pMesh->touch();
	}
	else
	{
		if (translation != NULL)
		{
			*translation = meshList->getManualMeshLoader(meshName)->getBoundingBox()->getCenter();
		}
	}

	Entity *pEntity = sceneMgr->createEntity(entityName, meshName);


	if (0) pEntity->setRenderQueueGroup(Ogre::RENDER_QUEUE_MAIN);

	return pEntity;
}


#endif